<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Log;

class OrderPrePopulateController extends Controller {

    private $shProductsToLip = [];

    public function __construct() {

        // $this->helper = new ResponseHelper();
    }

    /**
     * Get Shopify order details
     */
    public function  getShopifyOrderDetails(Request $request) {

        $orderId = !empty($request->get('id')) ? $request->get('id') : '';
        $responseData = [];
        $orderPrePopulatedData = [];
        $lineItemProperties = [];
        $shopifyProducts = [];

        if (empty($orderId)) {

            $errorMessage = 'Failed to get the order details from Shopify';
            $error = "Failed to get the orderId from the URL";
            $responseData = [
                "error" => $errorMessage
            ];
            Log::channel('customOrderCreationLog')->info($error);
            return view('order',compact('responseData'));
        }

        $shop = Auth::user();
        $orderDetails = $shop->api()->rest('GET', htmlentities('/admin/orders/'.$orderId.'.json'));

        $orderDetails = isset($orderDetails['body']['order']) ? 
        $orderDetails['body']['container']['order'] : '';

        if (!empty($orderDetails['error']) || !empty($orderDetails['errors']) || empty($orderDetails)) {

            $shError  = "Unknown error";
            if (!empty($orderDetails['error'])) {

                $shError = $orderDetails['error'];
            } else if (!empty($orderDetails['errors'])) {

                $shError = json_encode($orderDetails['errors']);
            } else if (empty($orderDetails)) {

                $shError = "Failed to get the order details from Shopify";
            }

            $errorMessage = 'Failed to get the order details from Shopify';
            $shErrorMessage = "Failed to get the order details from Shopify - OrderId: ".$orderId.", Error: ".$shError;
            $responseData = [
                "error" => $errorMessage
            ];
            Log::channel('customOrderCreationLog')->info($shErrorMessage);
            return view('order',compact('responseData'));
        }

        $shopifyProducts = $this->getShopifyProductDetails();

        $shLineItemProperties = isset($orderDetails['line_items'][0]['properties']) ? 
        $orderDetails['line_items'][0]['properties'] : '';

        if (empty($shopifyProducts) || empty($shLineItemProperties)) {

            $errorMessage = 'Failed to get the Shopify products or the order line items are empty';
            $shErrorMessage = "Failed to get the Shopify products or the order line items are empty - OrderId: ".$orderId;
            $responseData = [
                "error" => $errorMessage
            ];
            Log::channel('customOrderCreationLog')->info($shErrorMessage);
            return view('order',compact('responseData'));
        }

        $lineItemProperties = $this->getOrderLineItemPropertyDetails($shopifyProducts, $shLineItemProperties);

        $orderPrePopulatedData = [
            "email" => !empty($orderDetails['email']) ? trim($orderDetails['email']) : '',
            "firstName" => !empty($orderDetails['customer']['first_name']) ? 
            trim($orderDetails['customer']['first_name']) : '',
            "lastName" => !empty($orderDetails['customer']['last_name']) ? 
            trim($orderDetails['customer']['last_name']) : '',
            "shippingAddress" => !empty($orderDetails['shipping_address']) ? 
            $orderDetails['shipping_address'] : '',
            "billingAddress" => !empty($orderDetails['billing_address']) ? 
            $orderDetails['billing_address'] : '',
            "tags" => !empty($orderDetails['tags']) ? 
            $orderDetails['tags'] : '',
            "lip" => $lineItemProperties
        ];

        $responseData = [
            "prePopulateData" => json_encode($orderPrePopulatedData),
            "shProducts" => json_encode($this->shProductsToLip)
        ];

        return view('order',compact('responseData'));
    }

    /**
     * Get Shopify products
     */
    public function getShopifyProductDetails() {

        $shop    = Auth::user();
        $limit = 50;
        $previousLink = '';
        $nextLink = '';
        $shopifyProducts = [];
        $productVariants = [];

        do {
            if (empty($previousLink) && empty($nextLink)) {

                $productDetails = $shop->api()->rest('GET',
                '/admin/products.json',["published_status" => "published", "limit" => $limit, 
                "fields" => "id,title,handle,variants"]);
                $productDetails = (object)$productDetails;
                $previousLink = !empty($productDetails->link->previous) ? $productDetails->link->previous : '';
                $nextLink = !empty($productDetails->link->next) ? $productDetails->link->next : '';

            } else {

                $productDetails = $shop->api()->rest('GET',
                '/admin/products.json',["limit" => $limit,
                "fields" => 'id,title,handle,variants', "page_info" => $nextLink]);
                $productDetails = (object)$productDetails;
                $previousLink = !empty($productDetails->link->previous) ? $productDetails->link->previous : '';
                $nextLink = !empty($productDetails->link->next) ? $productDetails->link->next : '';
            }

            $productDetails = $productDetails->body->products;

            if (isset($productDetails)) {

                foreach ($productDetails as $products) {

                    $subscription_product = array('1809168695367', '4359186350162');

                    if (in_array($products->id, $subscription_product)) {
                        continue;
                    }

                    foreach ($products->variants as $variant) {

                        $productVariants[$variant->id] = [
                            'id' => $variant->id,
                            'product_id' => $variant->product_id,
                            'title' => strip_tags($variant->title),
                            'sku' => $variant->sku,
                            'price' => $variant->price
                        ];

                        $shopifyProducts[] = array(
                            "pid" => $products->id,
                            "vid" => $variant->id,
                            "sku" => $variant->sku,
                            "price" => $variant->price
                        );
                    }

                    $this->shProductsToLip['products'][$products->id] = [
                        'id' => $products->id,
                        'title' => strip_tags($products->title), 
                        'variants' => $productVariants
                    ];
                    $productVariants = [];
                }
            }

            $count = count($productDetails);
            if (empty($nextLink)) {
                break;
            }
        } while($limit == $count);

        return $shopifyProducts;
    }

    /**
     * Order line item property details formatting
     */
    public function getOrderLineItemPropertyDetails($shopifyProducts, $orderLips) {

        $orderLineItems = [];
        foreach ($orderLips as $orderLipKey => $orderLipValue) {

            $lipName = '';
            $lipValue = '';
            $lineItemExistingVid = '';
            $quantity = 0;
            $explodePropertyValue = '';
            $tempArray = [];

            $lipName = !empty($orderLipValue['name']) ? strtolower(trim($orderLipValue['name'])) : '';
            if (strpos($lipName, 'vitamin') !== false) {

                $lipValue = !empty($orderLipValue['value']) ? $orderLipValue['value'] : '';
                if (strpos($lipValue, '|') !== false) {
            
                    $explodePropertyValue = explode('|', $lipValue);

                    if (count($explodePropertyValue) == 5) {

                        $lineItemExistingVid = trim($explodePropertyValue[count($explodePropertyValue) - 2]);
                        $quantity = trim($explodePropertyValue[count($explodePropertyValue) - 3]);

                        foreach ($shopifyProducts as $shKey => $shValue) {

                            if (trim($shValue['vid']) == $lineItemExistingVid) {

                                $tempArray = [
                                    "lipName" => $orderLipValue['name'],
                                    "lipValue" => $lipValue,
                                    "price" => $shValue['price'],
                                    "pid" => $shValue['pid'],
                                    "vid" => $shValue['vid'],
                                    "quantity" => $quantity
                                ];
                                $orderLineItems[] = $tempArray;
                            }
                        }
                    }
                }
            }
        }

        return $orderLineItems;
    }

}
