<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\RechargeApi;
use Illuminate\Support\Facades\Auth;
use Log;

class SubscriptionController extends Controller
{

	private $rechargeObj;
	private $shop;
    private $domain; 
    
    /**
     * Create a new controller instance.
     *
     * @return void
    */
    public function __construct() {

        $this->rechargeObj   = new RechargeApi();
    }

    /**
     * Edit recharge subscription
     */
    public function editSubscription(Request $request) {

        $order_id = !empty($request->input('id')) ? trim($request->input('id')) : "";

        if (empty($order_id)) {
            Log::channel('subscriptionUpdateLog')->info("editSubscription: Failed to find the order Id");
            $message = "Failed to find the order Id";
            return view('subscription-error',["message" => $message]);
        }

        $shop = Auth::user();
        $shopify_order = $shop->api()->rest('GET', '/admin/orders/'.$order_id.'.json', []);
        
        $shopify_order = (object)$shopify_order;
            
        if (!isset($shopify_order->body->order) || empty($shopify_order->body->order->id)) {
            $errorMessage = "";
            if (isset($shopify_order->errors)) {
                $errorMessage = json_encode($shopify_order->errors);
            } else if (isset($shopify_order->error)) {
                $errorMessage = json_encode($shopify_order->error);
            } else {
                $errorMessage = "Failed to get the order details from Shopify";
            }
            Log::channel('subscriptionUpdateLog')->info("editSubscription: Failed to get the order details from Shopify, 
            OrderId:".$order_id.", Error:".$errorMessage);
            $message = "Failed to get the order details from Shopify";
            return view('subscription-error',["message" => $message]);
        }
        $shopify_order = $shopify_order->body;
        $order_details = ["id"=>$order_id, "name" => $shopify_order->order->name, "customer_name" => $shopify_order->order->customer->first_name, 
        "email" => $shopify_order->order->customer->email];
        $rechargeOrder = $this->rechargeObj->getOrderBySopifyOrderId($order_id);
            
        if (isset($rechargeOrder->errors) || isset($rechargeOrder->error) || empty($rechargeOrder->orders)) {
            $errorMessage = "";
            if (isset($rechargeOrder->errors)) {
                $errorMessage = json_encode($rechargeOrder->errors);
            } else if (isset($rechargeOrder->error)) {
                $errorMessage = json_encode($rechargeOrder->error);
            } else {
                $errorMessage = "Failed to get the order details from recharge";
            }
            Log::channel('subscriptionUpdateLog')->info("editSubscription: Failed to get the order details from recharge, 
            OrderId:".$order_id.", Error:".$errorMessage);
            $message = "Failed to load order details from recharge";
            return view('subscription-error',["message" => $message]);
        }
        
        $subscription_id = !empty($rechargeOrder->orders[0]->line_items[0]->subscription_id) ? 
        trim($rechargeOrder->orders[0]->line_items[0]->subscription_id) : "";

        $products  = $this->setProductDetails();
        $jsonProductDetails = json_encode($products); 
        $subscription =  $this->rechargeObj->getSubscription($subscription_id);
        if (isset($subscription->errors) || isset($subscription->error) || empty($subscription->subscription)) {
            $errorMessage = "";
            if (isset($subscription->errors)) {
                $errorMessage = json_encode($subscription->errors);
            } else if (isset($subscription->error)) {
                $errorMessage = json_encode($subscription->error);
            } else {
                $errorMessage = "Failed to get the subscription details from recharge";
            }
            Log::channel('subscriptionUpdateLog')->info("editSubscription: Failed to get the subscription details from recharge, 
            subscriptionId:".$subscription_id.", Error:".$errorMessage);
            $message = "Failed to load the subscription details from recharge";
            return view('subscription-error',["message" => $message]);
        }
        $lineitems = [];
        if (isset($subscription->subscription->properties) && !empty($subscription->subscription->properties)) {
            $lineitems = $subscription->subscription->properties;
        }
        
        return view('subscription-edit',['message'=>"",'subscriptionId'=>$subscription_id,'lineitems'=>$lineitems, 
        "products" => $products['variants'], "order" => $order_details],compact('jsonProductDetails'));
    }

    /**
     * Get Shopify products
     */
    private function setProductDetails() {

        $limit = 50;
        $nextLink = '';
        $previousLink = '';
        $shProductscount = 0;
        $shProducts  = [];
        $shProducts['products'] = [];
        $productVariants = [];
        $allVariants = [];
        
        $shop = Auth::user();

        do {
            try {

                if (empty($previousLink) && empty($nextLink)) {

                    $productDetails = $shop->api()->rest('GET', 
                    '/admin/products.json',["published_status" => "published", "fields" => 
                    "id,title,variants", "limit" => $limit]);
                    $productDetails = (object)$productDetails;
                    
                    $previousLink = !empty($productDetails->link->previous) ?
                    $productDetails->link->previous : '';
                    $nextLink = !empty($productDetails->link->next) ?
                    $productDetails->link->next : '';
                    
                } else {

                    $params = ["fields" => "id,title,variants", "limit" => $limit, 
                    'page_info' => $nextLink];
                    $productDetails = $shop->api()->rest('GET', '/admin/products.json',$params);
                    $productDetails = (object)$productDetails;
                    
                    $previousLink = !empty($productDetails->link->previous) ?
                    $productDetails->link->previous : '';
                    $nextLink = !empty($productDetails->link->next) ?
                    $productDetails->link->next : '';
                }   
            } catch(Exception $e) {
                Log::channel('subscriptionUpdateLog')->info('setProductDetails: An error occurred when fetching the products from Shopify');
                echo PHP_EOL . "An error occurred when fetching the products from Shopify and the error is -  ". $e->getMessage();
                die();
            }

            $productDetails = $productDetails->body;
            
            if (!empty($productDetails)) {
                
                $subscription_product = array('1809168695367','4359186350162');
                
                foreach ($productDetails->products as $products) {
                    if (in_array($products->id,$subscription_product)) {
                        continue;
                    }
                
                    foreach ($products->variants as $variants) {
                        $allVariants[$variants->id] = [
                            'id' => $variants->id ,
                            'product_id' => $variants ->product_id,
                            'title' => strip_tags($products->title),
                            'variant_title' => strip_tags($variants->title),
                            'sku' => $variants->sku,
                            'price' => $variants->price
                        ];
                        $productVariants[$variants->id] = [
                            'id' => $variants->id ,
                            'product_id' => $variants->product_id,
                            'title' => strip_tags($variants->title),
                            'sku' => $variants->sku,
                            'price' => $variants->price
                        ];
                    }
                    
                    $shProducts['products'][$products->id] = ['id'=>$products->id, 'title'=>strip_tags($products->title), 
                    'variants'=>$productVariants];
                    $productVariants = [];
                }
                $shProducts['variants'] = $allVariants;
            }
            $shProductscount = !empty($productDetails) ? count($productDetails->products) : 0;  
            if (empty($nextLink)) {
                break;
            }
        } while($limit == $shProductscount);
        
        return $shProducts;
    }

    /**
     * Save subscription changes to recharge
     */
    public function saveSubscription(Request $req) {
        
        $shop = Auth::user();
        
        $subscription_id = !empty($req['subscriptionId']) ? (int)$req['subscriptionId'] : "";
        if (empty($subscription_id)) {
            Log::channel('subscriptionUpdateLog')->info("saveSubscription: Failed to find the subscriptionId in the request");
            $errorResponse = "Failed to find the subscriptionId in the request";
            return view('subscription-error',["message" => $errorResponse]);
        }
        $order_id = !empty($req['orderId']) ? (int)$req['orderId'] : "";
        if (empty($order_id)) {
            Log::channel('subscriptionUpdateLog')->info("saveSubscription: Failed to find the orderId in the request");
            $errorResponse = "Failed to find the orderId in the request";
            return view('subscription-error',["message" => $errorResponse]);
        }
        $shopify_order = $shop->api()->rest('GET', '/admin/orders/'.$order_id.'.json', []);

        $shopify_order = (object)$shopify_order;

        if (!isset($shopify_order->body->order) || empty($shopify_order->body->order->id)) {
            $errorMessage = "";
            if (isset($shopify_order->errors)) {
                $errorMessage = json_encode($shopify_order->errors);
            } else if (isset($shopify_order->error)) {
                $errorMessage = json_encode($shopify_order->error);
            } else {
                $errorMessage = "Failed to get the order details from Shopify";
            }
            Log::channel('subscriptionUpdateLog')->info("saveSubscription: Failed to get the order details from Shopify, 
            OrderId:".$order_id.", Error:".$errorMessage);
            $errorResponse = "Failed to get the order details from Shopify";
            return view('subscription-error',["message" => $errorResponse]);
        }

        $shopify_order = $shopify_order->body;
        $order_details = ["id"=>$order_id,"name" => $shopify_order->order->name, "customer_name" => $shopify_order->order->customer->first_name, 
        "email" => $shopify_order->order->customer->email];

        $subscriptionRequest = $this->setSubscriptionRequest($req);
        if (!empty($subscriptionRequest['errorMessage'])) {
            $errorResponse = $subscriptionRequest['errorMessage'];
            return view('subscription-error',["message" => $errorResponse]);
        }
        $subscriptionRequest['subscription_id'] = $subscription_id;

        $result = $this->rechargeObj->updateSubscription(json_encode($subscriptionRequest));

        if (!empty($result['errorMessage'])) {
            $errorMessage = $result['errorMessage'];        
            Log::channel('subscriptionUpdateLog')->info("saveSubscription: Failed to update the subscription, 
            subscriptionId:".$subscription_id.", Error:".$errorMessage);
            $errorResponse = "Failed to update the subscription";
            return view('subscription-error',["message" => $errorResponse]);
        }
                
        $message = "";
        if (!empty($result['status'])) {
            $statusMessage = trim(strtolower($result['status']));
            if ($statusMessage == "failed") {
                $message = $result['errorMessage'];
            } else {
                $message = "Successfully updated the subscription"; 
            }
        }
        $shProducts  = $this->setProductDetails();
        $jsonProductDetails = json_encode($shProducts); 

        $subscription =  $this->rechargeObj->getSubscription($subscription_id);

        if (isset($subscription->errors) || isset($subscription->error) || empty($subscription->subscription)) {
            $errorMessage = "";
            if (isset($subscription->errors)) {
                $errorMessage = json_encode($subscription->errors);
            } else if (isset($subscription->error)) {
                $errorMessage = json_encode($subscription->error);
            } else {
                $errorMessage = "Failed to get the subscription details from recharge";
            }
            Log::channel('subscriptionUpdateLog')->info("saveSubscription: Failed to get the subscription details from recharge, 
            subscriptionId:".$subscription_id.", Error:".$errorMessage);
            // $errorResponse = "Failed to load the subscription details from recharge";
            // return view('subscription-error',["message" => $errorResponse]);
        }
        $lineitems = [];
        if (isset($subscription->subscription->properties) && !empty($subscription->subscription->properties)) {
            $lineitems = $subscription->subscription->properties;
        }
        
        return view('subscription-edit',['message' => $message, 'subscriptionId' => $subscription_id, 'lineitems' => $lineitems, 
        "products" => $shProducts['variants'], "order" => $order_details], compact('jsonProductDetails'));
    }
    
    /**
     * Set subscription update request
     */
    public function setSubscriptionRequest($reqObj) {

        $request = [];
        $lineProperties = [];
        $shProducts  = $this->setProductDetails();
        $i = 1;
        foreach ($reqObj['prod-variant-id'] as $key => $variantId) {
            
            if ($i < 10) {
                $propertyKey = 'Vitamin0'.$i;
            } else if ($i >= 10) {
                $propertyKey = 'Vitamin'.$i;
            }
            
            if (isset($shProducts['variants'][$variantId])) {

                if (isset($reqObj['prod-qty'][$key])) {

                    $lineProperties[] = ['name'=>$propertyKey,'value' => $shProducts['variants'][$variantId]['title'], 
                    'product_id' => $shProducts["variants"][$variantId]['product_id'], 'variant_id' => $variantId, 
                    'quantity' => $reqObj['prod-qty'][$key], 'dosage' => $shProducts["variants"][$variantId]['variant_title'] ];
                }
            }
            $i++;
        }
        $request['properties'] = $lineProperties;
        return $request;
    }

}
?>
